@@ -20,17 +20,17 @@ from sqlalchemy.schema import CreateSchema
 from multipledispatch import MDNotImplementedError
 
 import datashape
-from datashape import DataShape, Record, Option, var, dshape
+from datashape import DataShape, Record, Option, var, dshape, Map
 from datashape.predicates import isdimension, isrecord, isscalar
 from datashape import discover
 from datashape.dispatch import dispatch
 
-from toolz import (partition_all, keyfilter, memoize, valfilter,
-                   identity, concat, curry, merge)
+from toolz import (partition_all, keyfilter, memoize, valfilter, identity,
+                   concat, curry, merge)
 from toolz.curried import pluck, map
 
 from ..compatibility import unicode
-from ..utils import keywords, ignoring, iter_except
+from ..utils import keywords, ignoring, iter_except, filter_kwargs
 from ..convert import convert, ooc_types
 from ..append import append
 from ..resource import resource
@@ -167,7 +167,7 @@ def discover_typeengine(typ):
     if typ in revtypes:
         return dshape(revtypes[typ])[0]
     if type(typ) in revtypes:
-        return dshape(revtypes[type(typ)])[0]
+        return revtypes[type(typ)]
     if isinstance(typ, (sa.String, sa.Unicode)):
         return datashape.String(typ.length, typ.collation)
     else:
@@ -181,15 +181,27 @@ def discover_typeengine(typ):
     raise NotImplementedError("No SQL-datashape match for type %s" % typ)
 
 
+@discover.register(sa.ForeignKey, sa.sql.FromClause)
+def discover_foreign_key_relationship(fk, parent, parent_measure=None):
+    if fk.column.table is not parent:
+        parent_measure = discover(fk.column.table).measure
+    return {fk.parent.name: Map(discover(fk.parent.type), parent_measure)}
+
+
 @discover.register(sa.Column)
-def discover_sqlalchemy_column(col):
-    optionify = Option if col.nullable else identity
-    return Record([[col.name, optionify(discover(col.type))]])
+def discover_sqlalchemy_column(c):
+    meta = Option if c.nullable else identity
+    return Record([(c.name, meta(discover(c.type)))])
 
 
 @discover.register(sa.sql.FromClause)
 def discover_sqlalchemy_selectable(t):
+    ordering = dict((c, i) for i, c in enumerate(c for c in t.columns.keys()))
     records = list(sum([discover(c).parameters[0] for c in t.columns], ()))
+    fkeys = [discover(fkey, t, parent_measure=Record(records))
+             for fkey in t.foreign_keys]
+    for name, column in merge(*fkeys).items():
+        records[ordering[name]] = (name, column)
     return var * Record(records)
 
 
@@ -254,7 +266,26 @@ def discover_row_proxy(rp):
     return Record(list(zip(rp.keys(), map(discover, rp.values()))))
 
 
-def dshape_to_table(name, ds, metadata=None):
+def validate_foreign_keys(ds, foreign_keys):
+    # passed foreign_keys and column in dshape, but not a ForeignKey type
+    for field in foreign_keys:
+        if field not in ds.measure.names:
+            raise TypeError('Requested foreign key field %r is not a field in '
+                            'datashape %s' % (field, ds))
+    for field, typ in ds.measure.fields:
+        if field in foreign_keys and not isinstance(getattr(typ, 'ty', typ),
+                                                    Map):
+            raise TypeError('Foreign key %s passed in but not a Map '
+                            'datashape, got %s' % (field, typ))
+
+        if isinstance(typ, Map) and field not in foreign_keys:
+            raise TypeError('Map type %s found on column %s, but %r '
+                            "wasn't found in %s" %
+                            (typ, field, field, foreign_keys))
+
+
+def dshape_to_table(name, ds, metadata=None, foreign_keys=None,
+                    primary_key=None):
     """
     Create a SQLAlchemy table from a datashape and a name
 
@@ -267,9 +298,20 @@ def dshape_to_table(name, ds, metadata=None):
 
     if isinstance(ds, str):
         ds = dshape(ds)
+    if not isrecord(ds.measure):
+        raise TypeError('dshape measure must be a record type e.g., '
+                        '"{a: int64, b: int64}". Input measure is %r' %
+                        ds.measure)
     if metadata is None:
         metadata = sa.MetaData()
-    cols = dshape_to_alchemy(ds)
+    if foreign_keys is None:
+        foreign_keys = {}
+
+    validate_foreign_keys(ds, foreign_keys)
+
+    cols = dshape_to_alchemy(ds, primary_key=primary_key or frozenset())
+    cols.extend(sa.ForeignKeyConstraint([column_name], [referent])
+                for column_name, referent in foreign_keys.items())
     t = sa.Table(name, metadata, *cols, schema=metadata.schema)
     return attach_schema(t, t.schema)
 
@@ -280,16 +322,19 @@ def create_from_datashape(o, ds, **kwargs):
 
 
 @dispatch(sa.engine.base.Engine, DataShape)
-def create_from_datashape(engine, ds, schema=None, **kwargs):
+def create_from_datashape(engine, ds, schema=None, foreign_keys=None,
+                          primary_key=None, **kwargs):
     assert isrecord(ds), 'datashape must be Record type, got %s' % ds
     metadata = metadata_of_engine(engine, schema=schema)
     for name, sub_ds in ds[0].dict.items():
-        t = dshape_to_table(name, sub_ds, metadata=metadata)
+        t = dshape_to_table(name, sub_ds, metadata=metadata,
+                            foreign_keys=foreign_keys,
+                            primary_key=primary_key)
         t.create()
     return engine
 
 
-def dshape_to_alchemy(dshape):
+def dshape_to_alchemy(dshape, primary_key=frozenset()):
     """
 
     >>> dshape_to_alchemy('int')
@@ -306,20 +351,24 @@ def dshape_to_alchemy(dshape):
     """
     if isinstance(dshape, str):
         dshape = datashape.dshape(dshape)
+    if isinstance(dshape, Map):
+        return dshape_to_alchemy(dshape.key.measure, primary_key=primary_key)
     if isinstance(dshape, Option):
-        return dshape_to_alchemy(dshape.ty)
+        return dshape_to_alchemy(dshape.ty, primary_key=primary_key)
     if str(dshape) in types:
         return types[str(dshape)]
     if isinstance(dshape, datashape.Record):
         return [sa.Column(name,
-                          dshape_to_alchemy(typ),
+                          dshape_to_alchemy(getattr(typ, 'ty', typ),
+                                            primary_key=primary_key),
+                          primary_key=name in primary_key,
                           nullable=isinstance(typ[0], Option))
                 for name, typ in dshape.parameters[0]]
     if isinstance(dshape, datashape.DataShape):
         if isdimension(dshape[0]):
-            return dshape_to_alchemy(dshape[1])
+            return dshape_to_alchemy(dshape[1], primary_key=primary_key)
         else:
-            return dshape_to_alchemy(dshape[0])
+            return dshape_to_alchemy(dshape[0], primary_key=primary_key)
     if isinstance(dshape, datashape.String):
         fixlen = dshape[0].fixlen
         if fixlen is None:
@@ -360,6 +409,8 @@ def select_to_base(sel, dshape=None, **kwargs):
 @append.register(sa.Table, Iterator)
 def append_iterator_to_table(t, rows, dshape=None, **kwargs):
     assert not isinstance(t, type)
+    if not t.exists():
+        raise ValueError('table %r does not exist' % t.name)
     rows = iter(rows)
 
     # We see if the sequence is of tuples or dicts
@@ -394,6 +445,7 @@ def append_iterator_to_table(t, rows, dshape=None, **kwargs):
 def append_anything_to_sql_Table(t, c, **kwargs):
     for item in c:
         append(t, item, **kwargs)
+    return t
 
 
 @append.register(sa.Table, object)
@@ -456,10 +508,11 @@ def fullname(table, compiler):
 
 @resource.register(r'(.*sql.*|oracle|redshift)(\+\w+)?://.+')
 def resource_sql(uri, *args, **kwargs):
-    kwargs2 = keyfilter(keywords(sa.create_engine).__contains__, kwargs)
-    engine = create_engine(uri, **kwargs2)
-    ds = kwargs.get('dshape')
-    schema = kwargs.get('schema')
+    engine = create_engine(uri, **filter_kwargs(sa.create_engine, kwargs))
+    ds = kwargs.pop('dshape', None)
+    schema = kwargs.pop('schema', None)
+    foreign_keys = kwargs.pop('foreign_keys', None)
+    primary_key = kwargs.pop('primary_key', None)
 
     # we were also given a table name
     if args and isinstance(args[0], (str, unicode)):
@@ -467,11 +520,15 @@ def resource_sql(uri, *args, **kwargs):
         metadata = metadata_of_engine(engine, schema=schema)
 
         with ignoring(sa.exc.NoSuchTableError):
-            return attach_schema(sa.Table(table_name, metadata, autoload=True,
-                                          autoload_with=engine, schema=schema),
-                                 schema)
+            return attach_schema(
+                sa.Table(table_name, metadata, schema=schema,
+                         autoload_with=engine),
+                schema
+            )
         if ds:
-            t = dshape_to_table(table_name, ds, metadata=metadata)
+            t = dshape_to_table(table_name, ds, metadata=metadata,
+                                foreign_keys=foreign_keys,
+                                primary_key=primary_key)
             t.create()
             return t
         else:
@@ -479,7 +536,8 @@ def resource_sql(uri, *args, **kwargs):
 
     # We were not given a table name
     if ds:
-        create_from_datashape(engine, ds, schema=schema)
+        create_from_datashape(engine, ds, schema=schema,
+                              foreign_keys=foreign_keys)
     return engine
 
 
@@ -534,6 +592,8 @@ ooc_types.add(sa.Table)
 @dispatch(sa.Table)
 def drop(table):
     table.drop(table.bind, checkfirst=True)
+    if table.exists():
+        raise ValueError('table %r dropped but still exists' % table.name)
 
 
 @convert.register(pd.DataFrame, (sa.sql.Select, sa.sql.Selectable), cost=200.0)
@@ -547,6 +607,7 @@ def select_or_selectable_to_frame(el, **kwargs):
 
 
 class CopyToCSV(sa.sql.expression.Executable, sa.sql.ClauseElement):
+
     def __init__(self, element, path, delimiter=',', quotechar='"',
                  lineterminator=r'\n', escapechar='\\', header=True,
                  na_value=''):
@@ -561,6 +622,10 @@ class CopyToCSV(sa.sql.expression.Executable, sa.sql.ClauseElement):
         self.escapechar = escapechar
         self.na_value = na_value
 
+    @property
+    def bind(self):
+        return self.element.bind
+
 
 @compiles(CopyToCSV, 'postgresql')
 def compile_copy_to_csv_postgres(element, compiler, **kwargs):
@@ -604,7 +669,8 @@ def compile_copy_to_csv_mysql(element, compiler, **kwargs):
                            path=element.path,
                            delimiter=element.delimiter,
                            lineterminator=element.lineterminator,
-                           escapechar=element.escapechar.encode('unicode-escape').decode(),
+                           escapechar=element.escapechar.encode(
+                               'unicode-escape').decode(),
                            quotechar=element.quotechar)
 
 
@@ -636,7 +702,6 @@ def append_table_to_csv(csv, selectable, dshape=None, **kwargs):
     stmt = CopyToCSV(selectable, os.path.abspath(csv.path), **kwargs)
     with selectable.bind.begin() as conn:
         conn.execute(stmt)
-    csv.has_header = stmt.header
     return csv
 
 
