@@ -1,17 +1,25 @@
 from __future__ import absolute_import, division, print_function
 
-from datashape import dshape, Record
-from toolz import pluck, get, curry, keyfilter
-from contextlib import contextmanager
-from multiprocessing.pool import ThreadPool
 import inspect
 import datetime
 import tempfile
 import os
 import shutil
 import numpy as np
+
+from contextlib import contextmanager
+from multiprocessing.pool import ThreadPool
+
+from multipledispatch import Dispatcher
+
+from datashape import dshape, Record
+
+from toolz import pluck, get, curry, keyfilter
+
 from .compatibility import unicode
 
+sample = Dispatcher('sample')
+
 
 def iter_except(func, exception, first=None):
     """Call a `func` repeatedly until `exception` is raised. Optionally call
@@ -265,10 +273,6 @@ def into_path(*path):
     return os.path.join(os.path.dirname(odo.__file__), *path)
 
 
-from multipledispatch import Dispatcher
-sample = Dispatcher('sample')
-
-
 @curry
 def pmap(f, iterable):
     """Map `f` over `iterable` in parallel using a ``ThreadPool``.
@@ -367,3 +371,23 @@ def filter_kwargs(f, kwargs):
     6
     """
     return keyfilter(keywords(f).__contains__, kwargs)
+
+
+@curry
+def copydoc(from_, to):
+    """Copies the docstring from one function to another.
+
+    Paramaters
+    ----------
+    from_ : any
+        The object to copy the docstring from.
+    to : any
+        The object to copy the docstring to.
+
+    Returns
+    -------
+    to : any
+        ``to`` with the docstring from ``from_``
+    """
+    to.__doc__ = from_.__doc__
+    return to
